'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = requestStatusesPlugin;

var _actionReducersMap = require('./action-reducers-map');

var _actionReducersMap2 = _interopRequireDefault(_actionReducersMap);

var _initialResourceMetaState = require('../utils/initial-resource-meta-state');

var _initialResourceMetaState2 = _interopRequireDefault(_initialResourceMetaState);

var _warning = require('../utils/warning');

var _warning2 = _interopRequireDefault(_warning);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function requestStatusesPlugin(resourceType, options = {}) {
  const customInitialMeta = options.initialResourceMeta || {};
  const optionsToSend = {
    initialResourceMeta: _extends({}, _initialResourceMetaState2.default, customInitialMeta)
  };

  return function (state, action) {
    const reducer = _actionReducersMap2.default[action.type];

    if (process.env.NODE_ENV !== 'production') {
      if (reducer && !action.resourceName && !action.resourceType) {
        (0, _warning2.default)(`A resourceType was not included in an action with type ` + `"${action.type}". Without a resourceType, Redux Resource will ` + `not be able to update a slice of your store. For more information, refer to ` + `the guide on Request Actions: ` + `https://redux-resource.js.org/docs/requests/request-actions.html`, 'MISSING_RESOURCE_TYPE');
      }
    }

    const actionResourceType = action.resourceType || action.resourceName;

    if (actionResourceType !== resourceType) {
      return state;
    }

    const callActionReducer = reducer && actionResourceType === resourceType;
    return callActionReducer ? reducer(state, action, optionsToSend) : state;
  };
}